CREATE TABLE metadata
(
 var_name varchar(63) not null,
 var_value varchar(255) not null,
 PRIMARY KEY(var_name)
) ;
CREATE TABLE config
(
 var_name varchar(63) not null,
 var_value varchar(255) not null,
 is_visible integer not null default 1,
 need_server_restart integer not null default 0,
 PRIMARY KEY(var_name)
) ;
CREATE TABLE config_clob
(
 var_name varchar(63) not null,
 var_value text not null,
 PRIMARY KEY(var_name)
) ;
CREATE TABLE users
(
 id integer not null,
 guid varchar(36) not null,
 name varchar(63) not null,
 password varchar(48) not null,
 system_access integer not null,
 flags integer not null,
 full_name varchar(127) not null,
 description varchar(255) not null,
 grace_logins integer not null,
 auth_method integer not null,
 cert_mapping_method integer not null,
 cert_mapping_data text not null,
 PRIMARY KEY(id)
) ;
CREATE TABLE user_groups
(
 id integer not null,
 guid varchar(36) not null,
 name varchar(63) not null,
 system_access integer not null,
 flags integer not null,
 description varchar(255) not null,
 PRIMARY KEY(id)
) ;
CREATE TABLE user_group_members
(
 group_id integer not null,
 user_id integer not null,
 PRIMARY KEY(group_id,user_id)
) ;
CREATE TABLE user_profiles
(
 user_id integer not null,
 var_name varchar(255) not null,
 var_value text not null,
 PRIMARY KEY(user_id,var_name)
) ;
CREATE TABLE userdb_custom_attributes
(
 object_id integer not null,
 attr_name varchar(255) not null,
 attr_value text not null,
 PRIMARY KEY(object_id,attr_name)
) ;
CREATE TABLE object_properties
(
 object_id integer not null,
 name varchar(63) not null,
 status integer not null,
 is_deleted integer not null,
 is_system integer not null,
 image_id integer not null,
 last_modified integer not null,
 inherit_access_rights integer not null,
 status_calc_alg integer not null,
 status_prop_alg integer not null,
 status_fixed_val integer not null,
 status_shift integer not null,
 status_translation varchar(8) not null,
 status_single_threshold integer not null,
 status_thresholds varchar(8) not null,
 comments text not null,
 PRIMARY KEY(object_id)
) ;
CREATE TABLE object_custom_attributes
(
 object_id integer not null,
 attr_name varchar(127) not null,
 attr_value text not null,
 PRIMARY KEY(object_id,attr_name)
) ;
CREATE TABLE zones
(
 id integer not null,
 zone_guid integer not null,
 zone_type integer not null,
 controller_ip varchar(15) not null,
 PRIMARY KEY(id)
) ;
CREATE TABLE zone_ip_addr_list
(
 zone_id integer not null,
 ip_addr varchar(15) not null,
 PRIMARY KEY(zone_id,ip_addr)
) ;
CREATE TABLE nodes
(
 id integer not null,
 primary_ip varchar(15) not null,
 node_flags integer not null,
 snmp_version integer not null,
 community varchar(127) not null,
 usm_auth_password varchar(127) not null,
 usm_priv_password varchar(127) not null,
 usm_methods integer not null,
 snmp_oid varchar(255) not null,
 auth_method integer not null,
 secret varchar(64) not null,
 agent_port integer not null,
 status_poll_type integer not null,
 node_type integer not null,
 agent_version varchar(63) not null,
 platform_name varchar(63) not null,
 poller_node_id integer not null,
 zone_guid integer not null,
 proxy_node integer not null,
 snmp_proxy integer not null,
 required_polls integer not null,
 uname varchar(255) not null,
 use_ifxtable integer not null,
 PRIMARY KEY(id)
) ;
CREATE TABLE clusters
(
 id integer not null,
 cluster_type integer not null,
 PRIMARY KEY(id)
) ;
CREATE TABLE cluster_members
(
 cluster_id integer not null,
 node_id integer not null,
 PRIMARY KEY(cluster_id,node_id)
) ;
CREATE TABLE cluster_sync_subnets
(
 cluster_id integer not null,
 subnet_addr varchar(15) not null,
 subnet_mask varchar(15) not null,
 PRIMARY KEY(cluster_id,subnet_addr)
) ;
CREATE TABLE cluster_resources
(
 cluster_id integer not null,
 resource_id integer not null,
 resource_name varchar(255) not null,
 ip_addr varchar(15) not null,
 current_owner integer not null,
 PRIMARY KEY(cluster_id,resource_id)
) ;
CREATE TABLE subnets
(
 id integer not null,
 ip_addr varchar(15) not null,
 ip_netmask varchar(15) not null,
 zone_guid integer not null,
 synthetic_mask integer not null,
 PRIMARY KEY(id)
) ;
CREATE TABLE interfaces
(
 id integer not null,
 node_id integer not null,
 ip_addr varchar(15) not null,
 ip_netmask varchar(15) not null,
 if_type integer not null,
 if_index integer not null,
 mac_addr varchar(15) not null,
 synthetic_mask integer not null,
 required_polls integer not null,
 PRIMARY KEY(id)
) ;
CREATE TABLE network_services
(
 id integer not null,
 node_id integer not null,
 service_type integer not null,
 ip_bind_addr varchar(15) not null,
 ip_proto integer not null,
 ip_port integer not null,
 check_request text not null,
 check_responce text not null,
 poller_node_id integer not null,
 required_polls integer not null,
 PRIMARY KEY(id)
) ;
CREATE TABLE vpn_connectors
(
 id integer not null,
 node_id integer not null,
 peer_gateway integer not null,
 PRIMARY KEY(id)
) ;
CREATE TABLE vpn_connector_networks
(
 vpn_id integer not null,
 network_type integer not null,
 ip_addr varchar(15) not null,
 ip_netmask varchar(15) not null,
 PRIMARY KEY(vpn_id,ip_addr)
) ;
CREATE TABLE containers
(
 id integer not null,
 category integer not null,
 object_class integer not null,
 enable_auto_bind integer not null,
 auto_bind_filter text not null,
 PRIMARY KEY(id)
) ;
CREATE TABLE conditions
(
 id integer not null,
 activation_event integer not null,
 deactivation_event integer not null,
 source_object integer not null,
 active_status integer not null,
 inactive_status integer not null,
 script text not null,
 PRIMARY KEY(id)
) ;
CREATE TABLE cond_dci_map
(
 condition_id integer not null,
 sequence_number integer not null,
 dci_id integer not null,
 node_id integer not null,
 dci_func integer not null,
 num_polls integer not null,
 PRIMARY KEY(condition_id,sequence_number)
) ;
CREATE TABLE templates
(
 id integer not null,
 version integer not null,
 enable_auto_apply integer not null,
 apply_filter text not null,
 PRIMARY KEY(id)
) ;
CREATE TABLE dct_node_map
(
 template_id integer not null,
 node_id integer not null,
 PRIMARY KEY(template_id,node_id)
) ;
CREATE TABLE nsmap
(
 subnet_id integer not null,
 node_id integer not null,
 PRIMARY KEY(subnet_id,node_id)
) ;
CREATE TABLE container_members
(
 container_id integer not null,
 object_id integer not null,
 PRIMARY KEY(container_id,object_id)
) ;
CREATE TABLE container_categories
(
 category integer not null,
 name varchar(63),
 image_id integer not null,
 description text,
 PRIMARY KEY(category)
) ;
CREATE TABLE acl
(
 object_id integer not null,
 user_id integer not null,
 access_rights integer not null,
 PRIMARY KEY(object_id,user_id)
) ;
CREATE TABLE trusted_nodes
(
 source_object_id integer not null,
 target_node_id integer not null,
 PRIMARY KEY(source_object_id,target_node_id)
) ;
CREATE TABLE items
(
 item_id integer not null,
 node_id integer not null,
 template_id integer not null,
 template_item_id integer not null,
 name varchar(255) not null,
 description varchar(255) not null,
 source integer not null,
 datatype integer not null,
 polling_interval integer not null,
 retention_time integer not null,
 adv_schedule integer not null,
 status integer not null,
 delta_calculation integer not null,
 transformation text not null,
 instance varchar(255) not null,
 all_thresholds integer not null,
 resource_id integer not null,
 proxy_node integer not null,
 base_units integer not null,
 unit_multiplier integer not null,
 custom_units_name varchar(63) not null,
 perftab_settings text not null,
 PRIMARY KEY(item_id)
) ;
CREATE TABLE dci_schedules
(
 schedule_id integer not null,
 item_id integer not null,
 schedule varchar(255) not null,
 PRIMARY KEY(item_id,schedule_id)
) ;
CREATE TABLE raw_dci_values
(
 item_id integer not null,
 raw_value varchar(255) not null,
 last_poll_time integer not null,
 PRIMARY KEY(item_id)
) ;
CREATE INDEX idx_raw_dci_values_item_id ON raw_dci_values(item_id);
CREATE TABLE event_cfg
(
 event_code integer not null,
 event_name varchar(63) not null,
 severity integer not null,
 flags integer not null,
 message varchar(255) not null,
 description text not null,
 PRIMARY KEY(event_code)
) ;
CREATE TABLE event_log
(
 event_id bigint not null,
 event_code integer not null,
 event_timestamp integer not null,
 event_source integer not null,
 event_severity integer not null,
 event_message varchar(255) not null,
 root_event_id bigint not null,
 user_tag varchar(63) not null,
 PRIMARY KEY(event_id)
) ;
CREATE INDEX idx_event_log_event_timestamp ON event_log(event_timestamp);
CREATE TABLE actions
(
 action_id integer not null,
 action_name varchar(63) not null,
 action_type integer not null,
 is_disabled integer not null,
 rcpt_addr varchar(255) not null,
 email_subject varchar(255) not null,
 action_data text not null,
 PRIMARY KEY(action_id)
) ;
CREATE TABLE event_groups
(
 id integer not null,
 name varchar(63) not null,
 description varchar(255) not null,
 range_start integer not null,
 range_end integer not null,
 PRIMARY KEY(id)
) ;
CREATE TABLE event_group_members
(
 group_id integer not null,
 event_code integer not null,
 PRIMARY KEY(group_id,event_code)
) ;
CREATE TABLE event_policy
(
 rule_id integer not null,
 flags integer not null,
 comments text not null,
 script text not null,
 alarm_message varchar(255) not null,
 alarm_severity integer not null,
 alarm_key varchar(255) not null,
 alarm_timeout integer not null,
 alarm_timeout_event integer not null,
 situation_id integer not null,
 situation_instance varchar(255) not null,
 PRIMARY KEY(rule_id)
) ;
CREATE TABLE policy_source_list
(
 rule_id integer not null,
 object_id integer not null,
 PRIMARY KEY(rule_id,object_id)
) ;
CREATE TABLE policy_event_list
(
 rule_id integer not null,
 event_code integer not null,
 PRIMARY KEY(rule_id,event_code)
) ;
CREATE TABLE policy_action_list
(
 rule_id integer not null,
 action_id integer not null,
 PRIMARY KEY(rule_id,action_id)
) ;
CREATE TABLE policy_time_range_list
(
 rule_id integer not null,
 time_range_id integer not null,
 PRIMARY KEY(rule_id,time_range_id)
) ;
CREATE TABLE policy_situation_attr_list
(
 rule_id integer not null,
 situation_id integer not null,
 attr_name varchar(255) not null,
 attr_value varchar(255) not null,
 PRIMARY KEY(rule_id,situation_id,attr_name)
) ;
CREATE TABLE time_ranges
(
 time_range_id integer not null,
 wday_mask integer not null,
 mday_mask integer not null,
 month_mask integer not null,
 time_range varchar(255) not null,
 PRIMARY KEY(time_range_id)
) ;
CREATE TABLE deleted_objects
(
 object_id integer not null,
 object_class integer not null,
 name varchar(63) not null,
 ip_addr varchar(15) not null,
 ip_netmask varchar(15) not null,
 PRIMARY KEY(object_id)
) ;
CREATE TABLE thresholds
(
 threshold_id integer not null,
 item_id integer not null,
 sequence_number integer not null,
 fire_value varchar(255) not null,
 rearm_value varchar(255) not null,
 check_function integer not null,
 check_operation integer not null,
 parameter_1 integer not null,
 parameter_2 integer not null,
 event_code integer not null,
 rearm_event_code integer not null,
 current_state integer not null,
 repeat_interval integer not null,
 PRIMARY KEY(threshold_id)
) ;
CREATE INDEX idx_thresholds_item_id ON thresholds(item_id);
CREATE INDEX idx_thresholds_sequence ON thresholds(sequence_number);
CREATE TABLE alarms
(
 alarm_id integer not null,
 alarm_state integer not null,
 hd_state integer not null,
 hd_ref varchar(63) not null,
 creation_time integer not null,
 last_change_time integer not null,
 source_object_id integer not null,
 source_event_code integer not null,
 source_event_id bigint not null,
 message varchar(255) not null,
 original_severity integer not null,
 current_severity integer not null,
 repeat_count integer not null,
 alarm_key varchar(255) not null,
 ack_by integer not null,
 term_by integer not null,
 timeout integer not null,
 timeout_event integer not null,
 PRIMARY KEY(alarm_id)
) ;
CREATE TABLE alarm_notes
(
 note_id integer not null,
 alarm_id integer not null,
 change_time integer not null,
 user_id integer not null,
 note_text text not null,
 PRIMARY KEY(note_id)
) ;
CREATE INDEX idx_alarm_notes_alarm_id ON alarm_notes(alarm_id);
CREATE TABLE oid_to_type
(
 pair_id integer not null,
 snmp_oid varchar(255) not null,
 node_type integer not null,
 node_flags integer not null,
 PRIMARY KEY(pair_id)
) ;
CREATE TABLE snmp_trap_cfg
(
 trap_id integer not null,
 snmp_oid varchar(255) not null,
 event_code integer not null,
 user_tag varchar(63) not null,
 description varchar(255),
 PRIMARY KEY(trap_id)
) ;
CREATE TABLE snmp_trap_pmap
(
 trap_id integer not null,
 parameter integer not null,
 snmp_oid varchar(255),
 description varchar(255),
 PRIMARY KEY(trap_id,parameter)
) ;
CREATE TABLE agent_pkg
(
 pkg_id integer not null,
 pkg_name varchar(63),
 version varchar(31),
 platform varchar(63),
 pkg_file varchar(255),
 description varchar(255),
 PRIMARY KEY(pkg_id)
) ;
CREATE TABLE object_tools
(
 tool_id integer not null,
 tool_name varchar(255) not null,
 tool_type integer not null,
 tool_data text not null,
 description varchar(255) not null,
 flags integer not null,
 matching_oid varchar(255) not null,
 confirmation_text varchar(255) not null,
 PRIMARY KEY(tool_id)
) ;
CREATE TABLE object_tools_acl
(
 tool_id integer not null,
 user_id integer not null,
 PRIMARY KEY(tool_id,user_id)
) ;
CREATE TABLE object_tools_table_columns
(
 tool_id integer not null,
 col_number integer not null,
 col_name varchar(255),
 col_oid varchar(255),
 col_format integer,
 col_substr integer,
 PRIMARY KEY(tool_id,col_number)
) ;
CREATE TABLE syslog
(
 msg_id bigint not null,
 msg_timestamp integer not null,
 facility integer not null,
 severity integer not null,
 source_object_id integer not null,
 hostname varchar(127) not null,
 msg_tag varchar(32) not null,
 msg_text text not null,
 PRIMARY KEY(msg_id)
) ;
CREATE INDEX idx_syslog_msg_timestamp ON syslog(msg_timestamp);
CREATE TABLE script_library
(
 script_id integer not null,
 script_name varchar(255) not null,
 script_code text not null,
 PRIMARY KEY(script_id)
) ;
CREATE TABLE snmp_trap_log
(
 trap_id bigint not null,
 trap_timestamp integer not null,
 ip_addr varchar(15) not null,
 object_id integer not null,
 trap_oid varchar(255) not null,
 trap_varlist text not null,
 PRIMARY KEY(trap_id)
) ;
CREATE INDEX idx_snmp_trap_log_tt ON snmp_trap_log(trap_timestamp);
CREATE TABLE maps
(
 map_id integer not null,
 map_name varchar(255) not null,
 description text not null,
 root_object_id integer not null,
 PRIMARY KEY(map_id)
) ;
CREATE TABLE map_access_lists
(
 map_id integer not null,
 user_id integer not null,
 access_rights integer not null,
 PRIMARY KEY(map_id,user_id)
) ;
CREATE TABLE submaps
(
 map_id integer not null,
 submap_id integer not null,
 attributes integer not null,
 PRIMARY KEY(map_id,submap_id)
) ;
CREATE TABLE submap_object_positions
(
 map_id integer not null,
 submap_id integer not null,
 object_id integer not null,
 x integer not null,
 y integer not null,
 PRIMARY KEY(map_id,submap_id,object_id)
) ;
CREATE TABLE submap_links
(
 map_id integer not null,
 submap_id integer not null,
 object_id1 integer not null,
 object_id2 integer not null,
 link_type integer not null,
 port1 varchar(255) not null,
 port2 varchar(255) not null,
 PRIMARY KEY(map_id,submap_id,object_id1,object_id2)
) ;
CREATE TABLE agent_configs
(
 config_id integer not null,
 config_name varchar(255) not null,
 config_file text not null,
 config_filter text not null,
 sequence_number integer not null,
 PRIMARY KEY(config_id)
) ;
CREATE TABLE address_lists
(
 list_type integer not null,
 community_id integer not null,
 addr_type integer not null,
 addr1 varchar(15) not null,
 addr2 varchar(15) not null,
 PRIMARY KEY(list_type,community_id,addr_type,addr1,addr2)
) ;
CREATE INDEX idx_address_lists_list_type ON address_lists(list_type);
CREATE TABLE graphs
(
 graph_id integer not null,
 owner_id integer not null,
 name varchar(255) not null,
 config text not null,
 PRIMARY KEY(graph_id)
) ;
CREATE TABLE graph_acl
(
 graph_id integer not null,
 user_id integer not null,
 user_rights integer not null,
 PRIMARY KEY(graph_id,user_id)
) ;
CREATE TABLE certificates
(
 cert_id integer not null,
 cert_type integer not null,
 cert_data text not null,
 subject text not null,
 comments text not null,
 PRIMARY KEY(cert_id)
) ;
CREATE TABLE audit_log
(
 record_id integer not null,
 timestamp integer not null,
 subsystem varchar(32) not null,
 success integer not null,
 user_id integer not null,
 workstation varchar(63) not null,
 object_id integer not null,
 message text not null,
 PRIMARY KEY(record_id)
) ;
CREATE TABLE situations
(
 id integer not null,
 name varchar(127) not null,
 comments text not null,
 PRIMARY KEY(id)
) ;
CREATE TABLE snmp_communities
(
 id integer not null,
 community varchar(255) not null,
 PRIMARY KEY(id)
) ;
CREATE TABLE web_maps
(
 id integer not null,
 title varchar(63) not null,
 properties text not null,
 data text not null,
 PRIMARY KEY(id)
) ;
INSERT INTO metadata (var_name,var_value)
 VALUES ('SchemaVersion',93);
INSERT INTO metadata (var_name,var_value)
 VALUES ('Syntax','MYSQL');
INSERT INTO metadata (var_name,var_value)
 VALUES ('IDataTableCreationCommand','CREATE TABLE idata_%d (item_id integer not null,idata_timestamp integer not null,idata_value varchar(255) not null)');
INSERT INTO metadata (var_name,var_value)
 VALUES ('IDataIndexCreationCommand_0','CREATE INDEX idx_idata_%d_item_id ON idata_%d(item_id)');
INSERT INTO metadata (var_name,var_value)
 VALUES ('IDataIndexCreationCommand_1','CREATE INDEX idx_idata_%d_id_timestamp ON idata_%d(item_id,idata_timestamp)');
