/* $Id: nxcp.cpp 3751 2009-05-01 12:13:23Z alk $ */
/* 
** NetXMS - Network Management System
** NetXMS Foundation Library
** Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008 Victor Kirhenshtein
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
** File: nxcp.cpp
**
**/

#include "libnetxms.h"

#ifdef _WIN32
#define read	_read
#define close	_close
#endif


//
// Constants
//

#define FILE_BUFFER_SIZE      32768


//
// Get symbolic name for message code
//

TCHAR LIBNETXMS_EXPORTABLE *NXCPMessageCodeName(WORD wCode, TCHAR *pszBuffer)
{
   static const TCHAR *pszMsgNames[] =
   {
      _T("CMD_LOGIN"),
      _T("CMD_LOGIN_RESP"),
      _T("CMD_KEEPALIVE"),
      _T("CMD_SET_ALARM_HD_STATE"),
      _T("CMD_GET_OBJECTS"),
      _T("CMD_OBJECT"),
      _T("CMD_DELETE_OBJECT"),
      _T("CMD_MODIFY_OBJECT"),
      _T("CMD_OBJECT_LIST_END"),
      _T("CMD_OBJECT_UPDATE"),
      _T("CMD_GET_EVENTS"),
      _T("CMD_EVENTLOG_RECORDS"),
      _T("CMD_GET_CONFIG_VARLIST"),
      _T("CMD_SET_CONFIG_VARIABLE"),
      _T("CMD_GET_OBJECT_TOOLS"),
      _T("CMD_EXECUTE_ACTION"),
      _T("CMD_DELETE_CONFIG_VARIABLE"),
      _T("CMD_NOTIFY"),
      _T("CMD_TRAP"),
      _T("CMD_OPEN_EPP"),
      _T("CMD_CLOSE_EPP"),
      _T("CMD_SAVE_EPP"),
      _T("CMD_EPP_RECORD"),
      _T("CMD_LOCK_EVENT_DB"),
      _T("CMD_UNLOCK_EVENT_DB"),
      _T("CMD_SET_EVENT_INFO"),
      _T("CMD_EVENT_DB_RECORD"),
      _T("CMD_LOAD_EVENT_DB"),
      _T("CMD_REQUEST_COMPLETED"),
      _T("CMD_LOAD_USER_DB"),
      _T("CMD_USER_DATA"),
      _T("CMD_GROUP_DATA"),
      _T("CMD_USER_DB_EOF"),
      _T("CMD_UPDATE_USER"),
      _T("CMD_DELETE_USER"),
      _T("CMD_CREATE_USER"),
      _T("CMD_LOCK_USER_DB"),
      _T("CMD_UNLOCK_USER_DB"),
      _T("CMD_USER_DB_UPDATE"),
      _T("CMD_SET_PASSWORD"),
      _T("CMD_GET_NODE_DCI_LIST"),
      _T("CMD_NODE_DCI"),
      _T("CMD_NODE_DCI_LIST_END"),
      _T("CMD_DELETE_NODE_DCI"),
      _T("CMD_MODIFY_NODE_DCI"),
      _T("CMD_UNLOCK_NODE_DCI_LIST"),
      _T("CMD_SET_OBJECT_MGMT_STATUS"),
      _T("CMD_CREATE_NEW_DCI"),
      _T("CMD_GET_DCI_DATA"),
      _T("CMD_DCI_DATA"),
      _T("CMD_GET_MIB_TIMESTAMP"),
      _T("CMD_GET_MIB"),
      _T("CMD_TEST_DCI_TRANSFORMATION"),
      _T("CMD_OPEN_LPP"),
      _T("CMD_CREATE_OBJECT"),
      _T("CMD_GET_EVENT_NAMES"),
      _T("CMD_EVENT_NAME_LIST"),
      _T("CMD_BIND_OBJECT"),
      _T("CMD_UNBIND_OBJECT"),
      _T("CMD_GET_IMAGE_LIST"),
      _T("CMD_LOAD_IMAGE_FILE"),
      _T("CMD_IMAGE_LIST"),
      _T("CMD_IMAGE_FILE"),
      _T("CMD_AUTHENTICATE"),
      _T("CMD_GET_PARAMETER"),
      _T("CMD_GET_LIST"),
      _T("CMD_ACTION"),
      _T("CMD_GET_DEFAULT_IMAGE_LIST"),
      _T("CMD_DEFAULT_IMAGE_LIST"),
      _T("CMD_GET_ALL_ALARMS"),
      _T("CMD_GET_ALARM"),
      _T("CMD_ACK_ALARM"),
      _T("CMD_ALARM_UPDATE"),
      _T("CMD_ALARM_DATA"),
      _T("CMD_DELETE_ALARM"),
      _T("CMD_LOCK_ACTION_DB"),
      _T("CMD_UNLOCK_ACTION_DB"),
      _T("CMD_LOAD_ACTIONS"),
      _T("CMD_ACTION_DB_UPDATE"),
      _T("CMD_MODIFY_ACTION"),
      _T("CMD_CREATE_ACTION"),
      _T("CMD_DELETE_ACTION"),
      _T("CMD_ACTION_DATA"),
      _T("CMD_GET_CONTAINER_CAT_LIST"),
      _T("CMD_CONTAINER_CAT_DATA"),
      _T("CMD_DELETE_CONTAINER_CAT"),
      _T("CMD_CREATE_CONTAINER_CAT"),
      _T("CMD_MODIFY_CONTAINER_CAT"),
      _T("CMD_POLL_NODE"),
      _T("CMD_POLLING_INFO"),
      _T("CMD_COPY_DCI"),
      _T("CMD_WAKEUP_NODE"),
      _T("CMD_DELETE_EVENT_TEMPLATE"),
      _T("CMD_GENERATE_EVENT_CODE"),
      _T("CMD_LOCK_TRAP_CFG"),
      _T("CMD_UNLOCK_TRAP_CFG"),
      _T("CMD_CREATE_TRAP"),
      _T("CMD_MODIFY_TRAP"),
      _T("CMD_DELETE_TRAP"),
      _T("CMD_LOAD_TRAP_CFG"),
      _T("CMD_TRAP_CFG_RECORD"),
      _T("CMD_QUERY_PARAMETER"),
      _T("CMD_GET_SERVER_INFO"),
      _T("CMD_SET_DCI_STATUS"),
      _T("CMD_FILE_DATA"),
      _T("CMD_TRANSFER_FILE"),
      _T("CMD_UPGRADE_AGENT"),
      _T("CMD_GET_PACKAGE_LIST"),
      _T("CMD_PACKAGE_INFO"),
      _T("CMD_REMOVE_PACKAGE"),
      _T("CMD_INSTALL_PACKAGE"),
      _T("CMD_LOCK_PACKAGE_DB"),
      _T("CMD_UNLOCK_PACKAGE_DB"),
      _T("CMD_ABORT_FILE_TRANSFER"),
      _T("CMD_CHECK_NETWORK_SERVICE"),
      _T("CMD_GET_AGENT_CONFIG"),
      _T("CMD_UPDATE_AGENT_CONFIG"),
      _T("CMD_GET_PARAMETER_LIST"),
      _T("CMD_DEPLOY_PACKAGE"),
      _T("CMD_INSTALLER_INFO"),
      _T("CMD_GET_LAST_VALUES"),
      _T("CMD_APPLY_TEMPLATE"),
      _T("CMD_SET_USER_VARIABLE"),
      _T("CMD_GET_USER_VARIABLE"),
      _T("CMD_ENUM_USER_VARIABLES"),
      _T("CMD_DELETE_USER_VARIABLE"),
      _T("CMD_ADM_MESSAGE"),
      _T("CMD_ADM_REQUEST"),
      _T("CMD_CHANGE_IP_ADDR"),
      _T("CMD_REQUEST_SESSION_KEY"),
      _T("CMD_ENCRYPTED_MESSAGE"),
      _T("CMD_SESSION_KEY"),
      _T("CMD_REQUEST_ENCRYPTION"),
      _T("CMD_GET_ROUTING_TABLE"),
      _T("CMD_EXEC_TABLE_TOOL"),
      _T("CMD_TABLE_DATA"),
      _T("CMD_APPLY_LOG_POLICY"),
      _T("CMD_CHANGE_SUBSCRIPTION"),
      _T("CMD_GET_SYSLOG"),
      _T("CMD_SYSLOG_RECORDS"),
      _T("CMD_GET_LPP_LIST"),
      _T("CMD_OPEN_LOG_POLICY"),
      _T("CMD_CLOSE_LOG_POLICY"),
      _T("CMD_GET_OBJECT_TOOL_DETAILS"),
      _T("CMD_LOCK_OBJECT_TOOLS"),
      _T("CMD_UNLOCK_OBJECT_TOOLS"),
      _T("CMD_UPDATE_OBJECT_TOOL"),
      _T("CMD_DELETE_OBJECT_TOOL"),
      _T("CMD_SETUP_PROXY_CONNECTION"),
      _T("CMD_GENERATE_OBJECT_TOOL_ID"),
      _T("CMD_GET_SERVER_STATS"),
      _T("CMD_GET_SCRIPT_LIST"),
      _T("CMD_GET_SCRIPT"),
      _T("CMD_UPDATE_SCRIPT"),
      _T("CMD_DELETE_SCRIPT"),
      _T("CMD_RENAME_SCRIPT"),
      _T("CMD_GET_SESSION_LIST"),
      _T("CMD_KILL_SESSION"),
      _T("CMD_GET_TRAP_LOG"),
      _T("CMD_TRAP_LOG_RECORDS"),
      _T("CMD_START_SNMP_WALK"),
      _T("CMD_SNMP_WALK_DATA"),
      _T("CMD_GET_MAP_LIST"),
      _T("CMD_LOAD_MAP"),
      _T("CMD_SAVE_MAP"),
      _T("CMD_DELETE_MAP"),
      _T("CMD_RESOLVE_MAP_NAME"),
      _T("CMD_SUBMAP_DATA"),
      _T("CMD_UPLOAD_SUBMAP_BK_IMAGE"),
      _T("CMD_GET_SUBMAP_BK_IMAGE"),
      _T("CMD_GET_MODULE_LIST"),
      _T("CMD_UPDATE_MODULE_INFO"),
      _T("CMD_COPY_USER_VARIABLE"),
      _T("CMD_RESOLVE_DCI_NAMES"),
      _T("CMD_GET_MY_CONFIG"),
      _T("CMD_GET_AGENT_CFG_LIST"),
      _T("CMD_OPEN_AGENT_CONFIG"),
      _T("CMD_SAVE_AGENT_CONFIG"),
      _T("CMD_DELETE_AGENT_CONFIG"),
      _T("CMD_SWAP_AGENT_CONFIGS"),
      _T("CMD_TERMINATE_ALARM"),
      _T("CMD_GET_NXCP_CAPS"),
      _T("CMD_NXCP_CAPS"),
      _T("CMD_GET_OBJECT_COMMENTS"),
      _T("CMD_UPDATE_OBJECT_COMMENTS"),
      _T("CMD_ENABLE_AGENT_TRAPS"),
      _T("CMD_PUSH_DCI_DATA"),
      _T("CMD_GET_ADDR_LIST"),
      _T("CMD_SET_ADDR_LIST"),
      _T("CMD_RESET_COMPONENT"),
      _T("CMD_GET_DCI_EVENTS_LIST"),
      _T("CMD_CREATE_MGMT_PACK"),
      _T("CMD_INSTALL_MGMT_PACK"),
      _T("CMD_GET_TRAP_CFG_RO"),
		_T("CMD_SNMP_REQUEST"),
		_T("CMD_GET_DCI_INFO"),
		_T("CMD_GET_GRAPH_LIST"),
		_T("CMD_DEFINE_GRAPH"),
		_T("CMD_DELETE_GRAPH"),
		_T("CMD_GET_SYSTEM_DCI_LIST"),
		_T("CMD_ADD_CA_CERTIFICATE"),
		_T("CMD_DELETE_CERTIFICATE"),
		_T("CMD_GET_CERT_LIST"),
		_T("CMD_UPDATE_CERT_COMMENTS"),
		_T("CMD_QUERY_L2_TOPOLOGY"),
		_T("CMD_AUDIT_RECORD"),
		_T("CMD_GET_AUDIT_LOG"),
		_T("CMD_SEND_SMS"),
		_T("CMD_GET_COMMUNITY_LIST"),
		_T("CMD_UPDATE_COMMUNITY_LIST"),
		_T("CMD_GET_SITUATION_LIST"),
		_T("CMD_DELETE_SITUATION"),
		_T("CMD_CREATE_SITUATION"),
		_T("CMD_DEL_SITUATION_INSTANCE"),
		_T("CMD_UPDATE_SITUATION"),
		_T("CMD_SITUATION_DATA"),
		_T("CMD_SITUATION_CHANGE"),
		_T("CMD_CREATE_MAP"),
		_T("CMD_WEBMAP_ADD"),
		_T("CMD_WEBMAP_UPDATE_PROPS"),
		_T("CMD_WEBMAP_UPDATE_DATA"),
		_T("CMD_WEBMAP_DELETE"),
		_T("CMD_WEBMAP_GET_DATA"),
		_T("CMD_WEBMAP_GET_LIST"),
		_T("CMD_CONFIG_SET_CLOB"),
		_T("CMD_CONFIG_GET_CLOB"),
		_T("CMD_RENAME_MAP"),
		_T("CMD_CLEAR_DCI_DATA"),
		_T("CMD_GET_LICENSE"),
		_T("CMD_CHECK_LICENSE"),
		_T("CMD_RELEASE_LICENSE"),
		_T("CMD_ISC_CONNECT_TO_SERVICE"),
		_T("CMD_REGISTER_AGENT"),
		_T("CMD_GET_SERVER_FILE"),
		_T("CMD_FORWARD_EVENT")
   };

   if ((wCode >= CMD_LOGIN) && (wCode <= CMD_FORWARD_EVENT))
      _tcscpy(pszBuffer, pszMsgNames[wCode - CMD_LOGIN]);
   else
      _sntprintf(pszBuffer, 64, _T("CMD_UNKNOWN(%d)"), wCode);
   return pszBuffer;
}


//
// Receive raw CSCP message from network
// If pMsg is NULL, temporary buffer will be re-initialized
// Returns message size on success or:
//   0 if connection is closed
//  <0 on socket errors
//   1 if message is too large to fit in buffer (normal messages is at least 16
//     bytes long, so we never get length of 1 for valid message)
//     In this case, only message header will be copied into buffer
//   2 Message decryption failed
//   3 Receive timeout
//

int LIBNETXMS_EXPORTABLE RecvNXCPMessage(SOCKET hSocket, CSCP_MESSAGE *pMsg,
                                         CSCP_BUFFER *pBuffer, DWORD dwMaxMsgSize,
                                         CSCP_ENCRYPTION_CONTEXT **ppCtx, 
                                         BYTE *pDecryptionBuffer, DWORD dwTimeout)
{
   DWORD dwMsgSize = 0, dwBytesRead = 0, dwBytesToCopy;
   int iErr;
   BOOL bSkipMsg = FALSE;

   // Initialize buffer if requested
   if (pMsg == NULL)
   {
      pBuffer->dwBufSize = 0;
      pBuffer->dwBufPos = 0;
      return 0;
   }

   // Check if we have something in buffer
   if (pBuffer->dwBufSize > 0)
   {
      // Handle the case when entire message header have not been read into the buffer
      if (pBuffer->dwBufSize < CSCP_HEADER_SIZE)
      {
         // Most likely we are at the buffer end, so move content
         // to the beginning
         memmove(pBuffer->szBuffer, &pBuffer->szBuffer[pBuffer->dwBufPos], pBuffer->dwBufSize);
         pBuffer->dwBufPos = 0;

         // Receive new portion of data from the network 
         // and append it to existing data in buffer
			iErr = RecvEx(hSocket, &pBuffer->szBuffer[pBuffer->dwBufSize],
                       CSCP_TEMP_BUF_SIZE - pBuffer->dwBufSize, 0, dwTimeout);
         if (iErr <= 0)
            return (iErr == -2) ? 3 : iErr;
         pBuffer->dwBufSize += (DWORD)iErr;
      }

      // Get message size from message header and copy available 
      // message bytes from buffer
      dwMsgSize = ntohl(((CSCP_MESSAGE *)(&pBuffer->szBuffer[pBuffer->dwBufPos]))->dwSize);
      if (dwMsgSize > dwMaxMsgSize)
      {
         bSkipMsg = TRUE;  // Message is too large, will skip it
         memcpy(pMsg, &pBuffer->szBuffer[pBuffer->dwBufPos], CSCP_HEADER_SIZE);
      }
      dwBytesRead = min(dwMsgSize, pBuffer->dwBufSize);
      if (!bSkipMsg)
         memcpy(pMsg, &pBuffer->szBuffer[pBuffer->dwBufPos], dwBytesRead);
      pBuffer->dwBufSize -= dwBytesRead;
      pBuffer->dwBufPos = (pBuffer->dwBufSize > 0) ? (pBuffer->dwBufPos + dwBytesRead) : 0;
      if (dwBytesRead == dwMsgSize)
         goto decrypt_message;
   }

   // Receive rest of message from the network
   do
   {
      iErr = RecvEx(hSocket, pBuffer->szBuffer, CSCP_TEMP_BUF_SIZE, 0, dwTimeout);
      if (iErr <= 0)
         return (iErr == -2) ? 3 : iErr;

      if (dwBytesRead == 0 &&
		  iErr >= (sizeof(((CSCP_MESSAGE *)(pBuffer->szBuffer))->dwSize))) // New message?
      {
         dwMsgSize = ntohl(((CSCP_MESSAGE *)(pBuffer->szBuffer))->dwSize);
         if (dwMsgSize > dwMaxMsgSize)
         {
            bSkipMsg = TRUE;  // Message is too large, just skip it
            memcpy(pMsg, pBuffer->szBuffer, CSCP_HEADER_SIZE);
         }
      }
      dwBytesToCopy = min((DWORD)iErr, dwMsgSize - dwBytesRead);
      if (!bSkipMsg)
         memcpy(((char *)pMsg) + dwBytesRead, pBuffer->szBuffer, dwBytesToCopy);
      dwBytesRead += dwBytesToCopy;
   }
   while(dwBytesRead < dwMsgSize);
   
   // Check if we have something left in buffer
   if (dwBytesToCopy < (DWORD)iErr)
   {
      pBuffer->dwBufPos = dwBytesToCopy;
      pBuffer->dwBufSize = (DWORD)iErr - dwBytesToCopy;
   }

   // Check for encrypted message
decrypt_message:
   if ((!bSkipMsg) && (ntohs(pMsg->wCode) == CMD_ENCRYPTED_MESSAGE))
   {
      if ((*ppCtx != NULL) && (*ppCtx != PROXY_ENCRYPTION_CTX))
      {
         if (CSCPDecryptMessage(*ppCtx, (CSCP_ENCRYPTED_MESSAGE *)pMsg, pDecryptionBuffer))
         {
            dwMsgSize = ntohl(pMsg->dwSize);
         }
         else
         {
            dwMsgSize = 2;    // Decryption failed
         }
      }
      else
      {
         if (*ppCtx != PROXY_ENCRYPTION_CTX)
            dwMsgSize = 2;
      }
   }

   return bSkipMsg ? 1 : (int)dwMsgSize;
}


//
// Create CSCP message with raw data (MF_BINARY flag)
// If pBuffer is NULL, new buffer is allocated with malloc()
// Buffer should be of dwDataSize + CSCP_HEADER_SIZE + 8 bytes.
//

CSCP_MESSAGE LIBNETXMS_EXPORTABLE *CreateRawNXCPMessage(WORD wCode, DWORD dwId, WORD wFlags,
                                                        DWORD dwDataSize, void *pData, 
                                                        CSCP_MESSAGE *pBuffer)
{
   CSCP_MESSAGE *pMsg;
   DWORD dwPadding;

   if (pBuffer == NULL)
      pMsg = (CSCP_MESSAGE *)malloc(dwDataSize + CSCP_HEADER_SIZE + 8);
   else
      pMsg = pBuffer;

   // Message should be aligned to 8 bytes boundary
   dwPadding = (8 - ((dwDataSize + CSCP_HEADER_SIZE) % 8)) & 7;

   pMsg->wCode = htons(wCode);
   pMsg->wFlags = htons(MF_BINARY | wFlags);
   pMsg->dwId = htonl(dwId);
   pMsg->dwSize = htonl(dwDataSize + CSCP_HEADER_SIZE + dwPadding);
   pMsg->dwNumVars = htonl(dwDataSize);   // dwNumVars contains actual data size for binary message
   memcpy(pMsg->df, pData, dwDataSize);

   return pMsg;
}


//
// Send file over CSCP
//

BOOL LIBNETXMS_EXPORTABLE SendFileOverNXCP(SOCKET hSocket, DWORD dwId, const TCHAR *pszFile,
                                           CSCP_ENCRYPTION_CONTEXT *pCtx)
{
#ifndef UNDER_CE
   int hFile, iBytes;
#else
   FILE *hFile;
   int iBytes;
#endif
   DWORD dwPadding;
   BOOL bResult = FALSE;
   CSCP_MESSAGE *pMsg;
   CSCP_ENCRYPTED_MESSAGE *pEnMsg;

#ifndef UNDER_CE
   hFile = _topen(pszFile, O_RDONLY | O_BINARY);
   if (hFile != -1)
#else
   hFile = _tfopen(pszFile, _T("rb"));
   if (hFile != NULL)
#endif
   {
      // Allocate message and prepare it's header
      pMsg = (CSCP_MESSAGE *)malloc(FILE_BUFFER_SIZE + CSCP_HEADER_SIZE + 8);
      pMsg->dwId = htonl(dwId);
      pMsg->wCode = htons(CMD_FILE_DATA);
      pMsg->wFlags = htons(MF_BINARY);
      
      while(1)
      {
#ifndef UNDER_CE
         iBytes = read(hFile, pMsg->df, FILE_BUFFER_SIZE);
         if (iBytes < 0)
            break;
#else
         iBytes = fread(pMsg->df, 1, FILE_BUFFER_SIZE, hFile);
         if (ferror(hFile))
			{
            break;	// Read error
			}
#endif

         // Message should be aligned to 8 bytes boundary
         dwPadding = (8 - (((DWORD)iBytes + CSCP_HEADER_SIZE) % 8)) & 7;
         pMsg->dwSize = htonl((DWORD)iBytes + CSCP_HEADER_SIZE + dwPadding);
         pMsg->dwNumVars = htonl((DWORD)iBytes);   // dwNumVars contains actual data size for binary message
         if (iBytes < FILE_BUFFER_SIZE)
            pMsg->wFlags |= htons(MF_END_OF_FILE);

         if (pCtx != NULL)
         {
            pEnMsg = CSCPEncryptMessage(pCtx, pMsg);
            if (pEnMsg != NULL)
            {
               SendEx(hSocket, (char *)pEnMsg, ntohl(pEnMsg->dwSize), 0);
               free(pEnMsg);
            }
         }
         else
         {
            if (SendEx(hSocket, (char *)pMsg, (DWORD)iBytes + CSCP_HEADER_SIZE + dwPadding, 0) <= 0)
				{
					break;	// Send error
				}
         }

         if (iBytes < FILE_BUFFER_SIZE)
         {
            // End of file
            bResult = TRUE;
            break;
         }
      }

      free(pMsg);
#ifndef UNDER_CE
      close(hFile);
#else
      fclose(hFile);
#endif
   }

   // If file upload failed, send CMD_ABORT_FILE_TRANSFER
   if (!bResult)
   {
      CSCP_MESSAGE msg;

      msg.dwId = htonl(dwId);
      msg.wCode = htons(CMD_ABORT_FILE_TRANSFER);
      msg.wFlags = htons(MF_BINARY);
      msg.dwNumVars = 0;
      msg.dwSize = htonl(CSCP_HEADER_SIZE);
      if (pCtx != NULL)
      {
         pEnMsg = CSCPEncryptMessage(pCtx, &msg);
         if (pEnMsg != NULL)
         {
            SendEx(hSocket, (char *)pEnMsg, ntohl(pEnMsg->dwSize), 0);
            free(pEnMsg);
         }
      }
      else
      {
         SendEx(hSocket, (char *)&msg, CSCP_HEADER_SIZE, 0);
      }
   }

   return bResult;
}


//
// Get version of NXCP used by peer
//

BOOL LIBNETXMS_EXPORTABLE NXCPGetPeerProtocolVersion(SOCKET hSocket, int *pnVersion)
{
   CSCP_MESSAGE msg;
   CSCP_ENCRYPTION_CONTEXT *pDummyCtx = NULL;
   CSCP_BUFFER *pBuffer;
   BOOL bRet = FALSE;
   int nSize;

   msg.dwId = 0;
   msg.dwNumVars = 0;
   msg.dwSize = htonl(CSCP_HEADER_SIZE);
   msg.wCode = htons(CMD_GET_NXCP_CAPS);
   msg.wFlags = htons(MF_CONTROL);
   if (SendEx(hSocket, &msg, CSCP_HEADER_SIZE, 0) == CSCP_HEADER_SIZE)
   {
      pBuffer = (CSCP_BUFFER *)malloc(sizeof(CSCP_BUFFER));
      RecvNXCPMessage(0, NULL, pBuffer, 0, NULL, NULL, 0);
      nSize = RecvNXCPMessage(hSocket, &msg, pBuffer, CSCP_HEADER_SIZE, &pDummyCtx, NULL, 30000);
      if ((nSize == CSCP_HEADER_SIZE) &&
          (ntohs(msg.wCode) == CMD_NXCP_CAPS) &&
          (ntohs(msg.wFlags) & MF_CONTROL))
      {
         bRet = TRUE;
         *pnVersion = ntohl(msg.dwNumVars) >> 24;
      }
      else if ((nSize == 1) || (nSize == 3) || (nSize >= CSCP_HEADER_SIZE))
      {
         // We don't receive any answer or receive invalid answer - 
         // assume that peer doesn't understand CMD_GET_NXCP_CAPS message
         // and set version number to 1
         bRet = TRUE;
         *pnVersion = 1;
      }
      free(pBuffer);
   }
   return bRet;
}
